#!/bin/ksh
cd "$(cd "$(dirname "$0")"; pwd)"
# -----------------------------------------------------------------------------------
# ccdt.ksh
# 
# Builds a CCDT file from QMgr object backups
# 
# -----------------------------------------------------------------------------------

# Where & when
_BkupRoot=/var/mqm/cfg

# Fetch vars to construct log file name
_PROG=${0##*/} && _QMgr=${1} 
_DATE=`date "+%y%m%d-%H%M%S"`
_LOG=/var/mqm/log/$_PROG.$_DATE.out

# Execute a new shell with redirection of the log file
/bin/ksh -s > $_LOG 2>&1 <<EOFEXEC

export MQCERTLABL=ec2-user
export MQCHLLIB=/home/ec2-user
export MQSSLKEYR=/home/ec2-user/.ssh/key

print "
#-----------------------------------------------------------------
#      Start scanning queue manager configs at `date`
#-----------------------------------------------------------------
"
# Get the epoch time from 10 days ago
((Oldest=\$( date "+%s") - (10*24*60*60) ))

# Collect the matching SVRCONN channel defs
Cursor=0
ls -1 ${_BkupRoot} | {
  while read QMgr;do
    CONNAME=\$(grep "^\$QMgr/" ./connections.txt | wc -l | tr -d " ")
    if [[ \$CONNAME -lt 1 ]]; then
      print "\$QMgr: No connection entry found.  Skipping.\n"
      break
    fi
    print "\$QMgr: Connection entry found.  Checking for recent backups."
    Candidates=0
    GotOne=0
    ls -1r ${_BkupRoot}/\$QMgr/*.??????-??????.mqsc | {
      while read FileName;do
        BaseName=\$(basename \$FileName)
        YYMMDD=\${BaseName#*.}  # Strip QMgr name from front of string
        Age=\$(printf "%(%#)T" "\${YYMMDD:2:2}/\${YYMMDD:4:2}/\${YYMMDD:0:2}")
        [[ \$Age -lt \$Oldest ]] && break
        ((Candidates+=1))
        echo \$FileName
        GotOne=\$(grep -v '^*' \$FileName | grep 'CHLTYPE(SVRCONN' | wc -l | tr -d " ")
        [[ \$GotOne > 0 ]] && break
      done
      if [[ \$Candidates > 0 && \$GotOne == 0 ]]; then
        print "No SVRCONN channels found among most recent \$Candidates backups. Skipping.\n"
      else
        # Save off the channels so we can check for dup names before making CCDT
        grep -v '^*' \$FileName | grep 'CHLTYPE(SVRCONN' | {
          while read Channel;do
            # print "\t\$Cursor - \$Channel"
            Channels[\$Cursor]="\$Channel QMNAME(\$QMgr)"
            ((Cursor+=1))
          done
        }
      fi
    }
    echo
  done
}

#set -v
i=-1
LastChannel=""
Dupe=0
{
  while [[ i -lt \$Cursor ]]; do
    ((i+=1))
    print \${Channels[\$i]}
  done
} | sort | {
  while read Channel;do
    ThisChl=\$(echo \${Channel} | sed "s/.* CHANNEL('\([^']*\).*/\1/")
    QMNAME=\$(echo \${Channel} | sed "s/.* QMNAME(\([^)]*\).*/\1/")  
    SSLCIPH=\$(echo \${Channel} | sed "s/.* SSLCIPH('\([^']*\).*/\1/")
    CONNAME=\$(grep "^\$QMNAME/" ./connections.txt | cut -d / -f 3)  
    if [[ \$ThisChl == \$LastChannel ]];then
      ((Dupe+=1))
    else
      [[ \$Dupe -eq 0 ]] && print "DEFINE CHL(\$ThisChl) CHLTYPE(CLNTCONN) CONNAME('\$CONNAME') QMNAME(\$QMNAME) SSLCIPH('\$SSLCIPH')"
      LastChannel=\$ThisChl
      Dupe=0
    fi
  done
} | mqscx -n -t ./AMQCLCHL.TAB

print "CCDT processing complete at \$(date)

#-----------------------------------------------------------------
#                  E N D   O F   S C R I P T
#-----------------------------------------------------------------
"
EOFEXEC
cat $_LOG